#include <gpdef.h>
#include <gpstdlib.h>
#include <gpgraphic.h>
#include <gpgraphic16.h>

#include "../include/global.h"
#include "../include/graphics16.h"
#include "../include/okf.h"

#include "project.h"
#include "bitmaps.h"

#define ARROWUP     0x01
#define ARROWDOWN   0x02


// Global variables
unsigned char gtAlpha4[4][32][32];
unsigned char gtAlpha8[8][32][32];
unsigned char gtAlpha16[16][32][32];
unsigned char gtAlpha32[32][32][32];


// Implementations
void FadeOut(void)
{
  int tick;
  int i, j;
  unsigned short *ptrFront, *ptrBack, color;
  int red, green, blue;

  for (i=0; i <= 31; i++)
  {
    tick = GpTickCountGet();

    ptrFront = (unsigned short *) gtSurface[(giSurface+1) & 0x01].ptbuffer;
    ptrBack = (unsigned short *) gtSurface[giSurface].ptbuffer;
    for (j=0; j < GPC_LCD_WIDTH * GPC_LCD_HEIGHT; j++)
    {
      color = *ptrFront++;

      // Separate color components
      red   = GetRGBRed(color);
      green = GetRGBGreen(color);
      blue  = GetRGBBlue(color);

      if (red < 31)   red++;
      if (green < 31) green++;
      if (blue < 31)  blue++;

      *ptrBack++ = RGB(red, green, blue);
    }

    SurfaceFlip();
    
    while (GpTickCountGet() - tick < 50);
  }
}

void MessageBox(unsigned char *pTitle,
                int left, int top, int width, int height,
                unsigned char *pMessage)
{
  int pressedKeys, newKeys, oldKeys;
  OKFENGINE oldState;
  unsigned char pMemoText[2048];
  unsigned short *pStaticScreen;
  int t, h, lh, arrows;

  // Save actual state
  oldState = okf;

  // Prepare static screen
  //- background
  CopyBackScreen();
  //- help window
  OverlayColor16(&gtSurface[giSurface], left, top, width, height, RGB(31, 31, 31), 29);
  GpRectDraw16(&gtSurface[giSurface], left, top, left + width - 1, top + height - 1, RGB(31, 15, 0));
  GpRectFill16(NULL, &gtSurface[giSurface], left, top + 25, width, 1, RGB(31, 15, 0));
  //- window title
  OkfSetToDefaults();
  okf.font = 1;
  okf.effect.type = okfAddColor;
  okf.effect.color = RGB(31, 15, 0);
  okf.justify = okfCenter;
  OkfPrintAt(left + width / 2, top + 5, pTitle);
  //- save static screen
  pStaticScreen = SaveBackScreen();

  // Prepare text
  OkfSetToDefaults();
  okf.font = 3;
  okf.clip.x1 = left + 15;
  okf.clip.y1 = top + 38;
  okf.clip.x2 = left + width - 1 - 15;
  okf.clip.y2 = top + height - 1 - 13;
  OkfSplitLines(pMessage, okf.clip.x2 - okf.clip.x1 + 1, pMemoText);
  h = OkfGetTextHeight(pMemoText);
  if (h > okf.clip.y2 - okf.clip.y1 + 1)
    arrows = ARROWDOWN;
  else
    arrows = 0;
  lh = OkfGetCurrentFont()->pOkf->defaultHeight;

  while (GpKeyGet());
  t = 0;
  while (TRUE)
  {
    // Prepare screen
    //- background
    GpBitBlt16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT,
               (unsigned char *) pStaticScreen, 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT); 
    //- help text
    OkfPrintAt(okf.clip.x1, okf.clip.y1 + t, pMemoText);
    //- arrows
    GpTransBlt16(NULL, &gtSurface[giSurface],
                 left + width / 2 - arrows_size / 2, okf.clip.y1 - arrows_size - 4, arrows_size, arrows_size,
                 (unsigned char *) arrows_bitmap, 0, (arrows & ARROWUP) ? 0 : arrows_size,
                 arrows_width, arrows_height, arrows_transColor); 
    GpTransBlt16(NULL, &gtSurface[giSurface],
                 left + width / 2 - arrows_size / 2, okf.clip.y2 + 1 + 4, arrows_size, arrows_size,
                 (unsigned char *) arrows_bitmap, arrows_size, (arrows & ARROWDOWN) ? 0 : arrows_size,
                 arrows_width, arrows_height, arrows_transColor); 

    SurfaceFlip();

    // Get buttons state
    oldKeys = pressedKeys;
    newKeys = pressedKeys = GpKeyGet();

    // Filter buttons
    if ((newKeys & GPC_VK_LEFT) && (oldKeys & GPC_VK_LEFT))
      newKeys = newKeys & ~GPC_VK_LEFT;
    if ((newKeys & GPC_VK_RIGHT) && (oldKeys & GPC_VK_RIGHT))
      newKeys = newKeys & ~GPC_VK_RIGHT;

    // Manage buttons
    if (newKeys & (GPC_VK_FA | GPC_VK_FB | GPC_VK_SELECT))
    {
      break;
    }
    else if (arrows & ARROWUP && newKeys & GPC_VK_UP)
    {
      if ((t += 2) > 0)
        t = 0;
    }
    else if (arrows & ARROWDOWN && newKeys & GPC_VK_DOWN)
    {
      if ((t -= 2) < (okf.clip.y2 - okf.clip.y1 + 1) - h)
        t = (okf.clip.y2 - okf.clip.y1 + 1) - h;
    }
    else if (arrows & ARROWUP && newKeys & GPC_VK_LEFT)
    {
      if ((t += okf.clip.y2 - okf.clip.y1 - 1 - lh) > 0)
        t = 0;
    }
    else if (arrows & ARROWDOWN && newKeys & GPC_VK_RIGHT)
    {
      if ((t -= okf.clip.y2 - okf.clip.y1 - 1 - lh) < (okf.clip.y2 - okf.clip.y1 + 1) - h)
        t = (okf.clip.y2 - okf.clip.y1 + 1) - h;
    }

    arrows = 0;
    if (t < 0)
      arrows = arrows | ARROWUP;
    if (t > (okf.clip.y2 - okf.clip.y1 + 1) - h)
      arrows = arrows | ARROWDOWN;
  }

  while (GpKeyGet());

  // Restore state
  okf = oldState;
  ReleaseScreen(pStaticScreen);
}
